@@ -1,6 +1,7 @@
 from __future__ import print_function
 
 import unittest
+import io
 
 from svgelements import *
 
@@ -156,42 +157,42 @@ class TestElementShape(unittest.TestCase):
 
     def test_circle_equals_transformed_circle(self):
         shape1 = Circle(r=2)
-        shape2 = Circle() * "scale(2)"
+        shape2 = Circle().set('vector-effect', 'non-scaling-stroke') * "scale(2)"
         self.assertEqual(shape1, shape2)
         shape2.reify()
         self.assertEqual(shape1, shape2)
 
     def test_rect_equals_transformed_rect(self):
         shape1 = Rect(x=0, y=0, width=2, height=2)
-        shape2 = Rect(0, 0, 1, 1) * "scale(2)"
+        shape2 = Rect(0, 0, 1, 1).set('vector-effect', 'non-scaling-stroke') * "scale(2)"
         self.assertEqual(shape1, shape2)
         shape2.reify()
         self.assertEqual(shape1, shape2)
 
     def test_rrect_equals_transformed_rrect(self):
         shape1 = Rect(0, 0, 2, 2, 1, 1)
-        shape2 = Rect(0, 0, 1, 1, 0.5, 0.5) * "scale(2)"
+        shape2 = Rect(0, 0, 1, 1, 0.5, 0.5).set('vector-effect', 'non-scaling-stroke') * "scale(2)"
         self.assertEqual(shape1, shape2)
         shape2.reify()
         self.assertEqual(shape1, shape2)
 
     def test_line_equals_transformed_line(self):
         shape1 = SimpleLine(0, 0, 2, 2)
-        shape2 = SimpleLine(0, 0, 1, 1) * "scale(2)"
+        shape2 = SimpleLine(0, 0, 1, 1).set('vector-effect', 'non-scaling-stroke') * "scale(2)"
         self.assertEqual(shape1, shape2)
         shape2.reify()
         self.assertEqual(shape1, shape2)
 
     def test_polyline_equals_transformed_polyline(self):
         shape1 = Polyline(0, 0, 2, 2)
-        shape2 = Polyline(0, 0, 1, 1) * "scale(2)"
+        shape2 = Polyline(0, 0, 1, 1).set('vector-effect', 'non-scaling-stroke') * "scale(2)"
         self.assertEqual(shape1, shape2)
         shape2.reify()
         self.assertEqual(shape1, shape2)
 
     def test_polygon_equals_transformed_polygon(self):
         shape1 = Polyline(0, 0, 2, 2)
-        shape2 = Polyline(0, 0, 1, 1) * "scale(2)"
+        shape2 = Polyline(0, 0, 1, 1).set('vector-effect', 'non-scaling-stroke') * "scale(2)"
         self.assertEqual(shape1, shape2)
         shape2.reify()
         self.assertEqual(shape1, shape2)
@@ -203,7 +204,7 @@ class TestElementShape(unittest.TestCase):
 
     def test_polyline_closed_equals_transformed_polygon(self):
         shape1 = Path(Polyline(0, 0, 2, 2)) + "z"
-        shape2 = Polygon(0, 0, 1, 1) * "scale(2)"
+        shape2 = Polygon(0, 0, 1, 1).set('vector-effect', 'non-scaling-stroke') * "scale(2)"
         self.assertEqual(shape1, shape2)
 
     def test_path_plus_shape(self):
@@ -357,8 +358,8 @@ class TestElementShape(unittest.TestCase):
 
     def test_rect_reify(self):
         """Reifying a rotated rect."""
-        test_reification(self, Rect())
-        test_reification(self, Rect(2, 2, 4, 4))
+        reification_checks(self, Rect())
+        reification_checks(self, Rect(2, 2, 4, 4))
 
         shape = Rect() * "rotate(-90) translate(20,0)"
         t = Rect(0, -20, 1, 1)
@@ -367,36 +368,216 @@ class TestElementShape(unittest.TestCase):
 
     def test_circle_reify(self):
         """Reifying a rotated circle."""
-        test_reification(self, Circle())
-        test_reification(self, Circle(2, 2, 4, 4))
+        reification_checks(self, Circle())
+        reification_checks(self, Circle(2, 2, 4, 4))
 
     def test_ellipse_reify(self):
         """Reifying a rotated ellipse."""
-        test_reification(self, Ellipse(rx=1, ry=2))
-        test_reification(self, Ellipse(2, 2, 5, 8))
+        reification_checks(self, Ellipse(rx=1, ry=2))
+        reification_checks(self, Ellipse(2, 2, 5, 8))
 
     def test_polyline_reify(self):
         """Reifying a rotated polyline."""
-        test_reification(self, Polyline("0,0 1,1 2,2"))
-        test_reification(self, Polyline("0,0 1,1 2,0"))
+        reification_checks(self, Polyline("0,0 1,1 2,2"))
+        reification_checks(self, Polyline("0,0 1,1 2,0"))
 
     def test_polygon_reify(self):
         """Reifying a rotated polygon."""
-        test_reification(self, Polygon("0,0 1,1 2,2"))
-        test_reification(self, Polygon("0,0 1,1 2,0"))
+        reification_checks(self, Polygon("0,0 1,1 2,2"))
+        reification_checks(self, Polygon("0,0 1,1 2,0"))
 
     def test_line_reify(self):
         """Reifying a rotated line."""
-        test_reification(self, SimpleLine(0, 0, 1, 1))
-        test_reification(self, SimpleLine(2, 2, 1, 0))
+        reification_checks(self, SimpleLine(0, 0, 1, 1))
+        reification_checks(self, SimpleLine(2, 2, 1, 0))
 
     def test_path_reify(self):
         """Reifying a path."""
-        test_reification(self, Path("M0,0L1,1L1,0z"))
-        test_reification(self, Path("M100,100L70,70L45,0z"))
+        reification_checks(self, Path("M0,0L1,1L1,0z"))
+        reification_checks(self, Path("M100,100L70,70L45,0z"))
+
+    def test_shapes_degenerate(self):
+        """Testing Degenerate Shapes"""
+        self.assertEqual(Rect(0, 0, 0, 100).d(), '')
+        self.assertEqual(Rect(0, 0, 100, 0).d(), '')
+        self.assertEqual(Circle(0, 0, 0).d(), '')
+        self.assertEqual(Ellipse(0,0,0,100).d(), '')
+        self.assertEqual(Ellipse(0, 0, 100, 0).d(), '')
+        self.assertEqual(Polygon(points='').d(), '')
+
+    def test_issue_95(self):
+        """Testing Issue 95 stroke-width"""
+        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
+                        <svg>
+                        <ellipse style="stroke:#fc0000;stroke-width:1;fill:none" cx="0" cy="0" rx="1" ry="1" transform="scale(100) rotate(-90,0,0)"/>
+                        <rect style="stroke:#fc0000;stroke-width:1;fill:none" x="0" y="0" width="10" height="10" transform="scale(100) rotate(-90,0,0)"/>
+                        </svg>''')
+        m = SVG.parse(q)
+        ellipse = m[0]
+        for i in range(5):
+            ellipse = ellipse.reify()
+        self.assertEqual(ellipse.stroke_width, 1.0)
+
+        rect = m[1]
+        for i in range(5):
+            rect = rect.reify()
+        self.assertEqual(rect.stroke_width, 1.0)
+
+    def test_issue_99(self):
+        """Test Issue of inverted circle reified location"""
+        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
+                        <svg
+                            width="82.475mm"
+                            height="35.215mm"
+                            viewBox="24.766026 -242.607513 82.475082 35.214996"
+                            version="1.1"
+                        >
+                        <circle
+                            transform="scale(1,-1)"
+                            style="opacity:0.99;fill:none;stroke:#ff0000;stroke-width:0.0264584;stroke-miterlimit:4;stroke-dasharray:none"
+                            r="2"
+                            cx="100.41245"
+                            cy="211.59723"
+                            id="circle2" /></svg>
+        ''')
+        m = SVG.parse(q, reify=False)
+        q = copy(m[0])
+        r = copy(m[0])
+        self.assertEqual(q, r)
+        q.reify()
+        r = Path(r)
+        q = Path(q)
+        self.assertEqual(q, r)
+        r.reify()
+        q.reify()
+        self.assertEqual(q, r)
+
+    def test_issue_99b(self):
+        """Test Issue of double inverted circle reified location"""
+        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
+                        <svg
+                            width="82.475mm"
+                            height="35.215mm"
+                            viewBox="24.766026 -242.607513 82.475082 35.214996"
+                            version="1.1"
+                        >
+                        <circle
+                            transform="scale(-1,-1)"
+                            style="opacity:0.99;fill:none;stroke:#ff0000;stroke-width:0.0264584;stroke-miterlimit:4;stroke-dasharray:none"
+                            r="2"
+                            cx="100.41245"
+                            cy="211.59723"
+                            id="circle2" /></svg>
+        ''')
+        m = SVG.parse(q, reify=False)
+        q = copy(m[0])
+        r = copy(m[0])
+        self.assertEqual(q, r)
+        q.reify()
+        r = Path(r)
+        q = Path(q)
+        self.assertEqual(q, r)
+        r.reify()
+        q.reify()
+        self.assertEqual(q, r)
+
+    def test_issue_99c(self):
+        """Test Issue of inverted rect reified location"""
+        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
+                        <svg
+                            width="82.475mm"
+                            height="35.215mm"
+                            viewBox="24.766026 -242.607513 82.475082 35.214996"
+                            version="1.1"
+                        >
+                        <rect
+                            transform="scale(1,-1)"
+                            style="opacity:0.99;fill:none;stroke:#ff0000;stroke-width:0.0264584;stroke-miterlimit:4;stroke-dasharray:none"
+                            rx="2"
+                            x="100.41245"
+                            y="211.59723"
+                            width="100"
+                            height="100"
+                            id="circle2" /></svg>
+        ''')
+        m = SVG.parse(q, reify=False)
+        q = copy(m[0])
+        r = copy(m[0])
+        self.assertEqual(q, r)
+        q.reify()
+        r = Path(r)
+        q = Path(q)
+        self.assertEqual(q, r)
+        r.reify()
+        q.reify()
+        self.assertEqual(q, r)
+
+    def test_issue_99d(self):
+        """Test Issue of double inverted rect reified location"""
+        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
+                        <svg
+                            width="82.475mm"
+                            height="35.215mm"
+                            viewBox="24.766026 -242.607513 82.475082 35.214996"
+                            version="1.1"
+                        >
+                        <rect
+                            transform="scale(-1,-1)"
+                            style="opacity:0.99;fill:none;stroke:#ff0000;stroke-width:0.0264584;stroke-miterlimit:4;stroke-dasharray:none"
+                            rx="2"
+                            x="100.41245"
+                            y="211.59723"
+                            width="100"
+                            height="100"
+                            id="circle2" /></svg>
+        ''')
+        m = SVG.parse(q, reify=False)
+        q = copy(m[0])
+        r = copy(m[0])
+        self.assertEqual(q, r)
+        q.reify()
+        r = Path(r)
+        q = Path(q)
+        self.assertEqual(q, r)
+        r.reify()
+        q.reify()
+        self.assertEqual(q, r)
+
+    def test_issue_104(self):
+        """Testing Issue 104 degenerate parsing"""
+        q = io.StringIO(u'''<?xml version="1.0" encoding="utf-8" ?>
+                        <svg>
+                        <polygon points=""/>
+                        <polygon/>
+                        <rect x="0" y="0" width="0" height="10"/>
+                        <circle cx="0" cy="0" r="0"/>
+                        </svg>''')
+        m = SVG.parse(q)
+        self.assertEqual(len(m), 0)
+
+    def test_shape_npoints(self):
+        import numpy as np
+        shapes = [
+            Rect(10, 20, 300, 340),
+            Circle(10, 10, 5),
+            Ellipse(50, 50, 30, 20),
+            Polygon(points=((10, 10), (20, 30), (50, 20))),
+            Polyline(points=((10, 10), (20, 30), (50, 20), (100, 120))),
+        ]
+
+        for shape in shapes:
+            pos = np.linspace(0, 1, 1000)
+            # with disable_numpy():
+            pts1 = shape.npoint(pos)  # Test rendered worthless.
+
+            pts2 = shape.npoint(pos)
+
+            for p, p1, p2 in zip(pos, pts1, pts2):
+                self.assertEqual(shape.point(p), Point(p1))
+                self.assertEqual(Point(p1), Point(p2))
 
 
-def test_reification(test, shape):
+def reification_checks(test, shape):
     correct_reify(test, shape * "rotate(-90) translate(20,0)")
     correct_reify(test, shape * "rotate(12turn)")
     correct_reify(test, shape * "translate(20,0)")
